/***************************************************************************/
/*                                                                         */
/*  File: Shapes.cpp                                                       */
/*  Date: 10-03-07                                                         */
/*                                                                         */
/***************************************************************************/
/*
	See .h file
*/
/***************************************************************************/


#include <windows.h>

#include <stdio.h>					// sprintf(..)
#include <d3dx9.h>					// Direct X Header Files
#pragma comment(lib, "d3d9.lib")	// Direct X Library Files	
#pragma comment(lib, "d3dx9.lib")

#include "Shapes.h"


/***************************************************************************/
/*                                                                         */
/*  Name: DrawCube()                                                       */
/*                                                                         */
/*  Simple function to draw a 3D Cube in the world, for debugging/testing  */
/*                                                                         */
/***************************************************************************/

void DrawCube(IDirect3DDevice9  * pDevice, 
			  const D3DXVECTOR3& pos, 
			  const D3DXVECTOR3& rot, 
			  const D3DXVECTOR3& dim,
			  const D3DXMATRIX&  world /*=D3DXMATRIX(1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1)*/)
{

  	struct TLVERTEX
	{
		float x,y,z;
		unsigned int colour;
		enum{ FVF_TLVERTEX = D3DFVF_XYZ | D3DFVF_DIFFUSE };
	};

	pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);
	//pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	pDevice->SetTextureStageState(0,D3DTSS_COLOROP,	D3DTOP_SELECTARG1);
	pDevice->SetTextureStageState(0,D3DTSS_COLORARG1,	D3DTA_DIFFUSE);
	float w = 0.5f;
	float h = 0.5f;
	float d = 0.5f;
	DWORD col = 0xffff00ff;

	TLVERTEX Vertex[36] = 
	{
		//   x     y      z       colour
		// Front
		{   -w,   -h,	-d,    0xffff0000},
		{   +w,   +h,	-d,    0xffff0000},
		{   -w,   +h,	-d,    0xffff0000},

		{   -w,   -h,	-d,    0xffff0000},
		{   +w,   -h,	-d,    0xffff0000},
		{   +w,   +h,	-d,    0xffff0000},

		// Back
		{   -w,   -h,	+d,    0xff00ff00},
		{   -w,   +h,	+d,    0xff00ff00},
		{   +w,   +h,	+d,    0xff00ff00},

		{   -w,   -h,	+d,    0xff00ff00},
		{   +w,   +h,	+d,    0xff00ff00},
		{   +w,   -h,	+d,    0xff00ff00},

		// Top
		{   -w,   +h,	-d,    0xff0000ff},
		{   +w,   +h,	+d,    0xff0000ff},
		{   -w,   +h,	+d,    0xff0000ff},
					  
		{   -w,   +h,	-d,    0xff0000ff},
		{   +w,   +h,	-d,    0xff0000ff},
		{   +w,   +h,	+d,    0xff0000ff},

		// Bottom
		{   -w,   -h,	-d,    0xffffff0f},
		{   -w,   -h,	+d,    0xffffff0f},
		{   +w,   -h,	+d,    0xffffff0f},
					  
		{   -w,   -h,	-d,    0xffffff00},
		{   +w,   -h,	+d,    0xffffff00},
		{   +w,   -h,	-d,    0xffffff00},

		// Left
		{   -w,   -h,	-d,    0xffffffff},
		{   -w,   +h,	-d,    0xffffffff},
		{   -w,   +h,	+d,    0xffffffff},
				  
		{   -w,   -h,	-d,    0xffffffff},
		{   -w,   +h,	+d,    0xffffffff},
		{   -w,   -h,	+d,    0xffffffff},

		// Right
		{   +w,   -h,	-d,    0xff000000},
		{   +w,   +h,	+d,    0xff000000},
		{   +w,   +h,	-d,    0xff000000},
			  
		{   +w,   -h,	-d,    0xff000000},
		{   +w,   -h,	+d,    0xff000000},
		{   +w,   +h,	+d,    0xff000000},
	};
	pDevice->SetTexture( 0, NULL);

		
	#if(DIRECT3D_VERSION >= 0x0900)
		pDevice->SetFVF( TLVERTEX::FVF_TLVERTEX );
	#else
		pDevice->SetVertexShader( TLVERTEX::FVF_TLVERTEX );
	#endif // DIRECT3D_VERSION

	D3DXMATRIX matMove, matScale, matRot, matWorld;

	D3DXMatrixTranslation(&matMove, pos.x, pos.y, pos.z);
	D3DXMatrixScaling(&matScale, dim.x, dim.y, dim.z);
	D3DXMatrixRotationYawPitchRoll(&matRot, rot.y, rot.x, rot.z);

	matWorld = world * matRot * matScale * matMove;
	pDevice->SetTransform(D3DTS_WORLD, &matWorld);

	HRESULT hr = 
	pDevice->DrawPrimitiveUP( D3DPT_TRIANGLELIST, 12, Vertex, sizeof( TLVERTEX ) );

}//End DrawCube(..)


/***************************************************************************/
/*                                                                         */
/*  Name: DrawLine()                                                       */
/*                                                                         */
/*  Simple function to draw a line in the world, for debugging/testing     */
/*                                                                         */
/***************************************************************************/

void DrawLine(IDirect3DDevice9  * pDevice, 
			  const D3DXVECTOR3& from,
		      const D3DXVECTOR3& to,
			  unsigned int col,
			  const D3DXMATRIX& mat /*= D3DXMATRIX(1,0,0,0, 0,1,0,0,  0,0,1,0,  0,0,0,1)*/)
{
		struct TLVERTEX
		{
			float x,y,z;
			unsigned int colour;
			enum { FVF_VERTEX = (D3DFVF_XYZ | D3DFVF_DIFFUSE) };
		};

		pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
		pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
		pDevice->SetTextureStageState(0,D3DTSS_COLOROP,	D3DTOP_SELECTARG1);
		pDevice->SetTextureStageState(0,D3DTSS_COLORARG1,	D3DTA_DIFFUSE);
		
		TLVERTEX Vertex[2] = 
		{
			//   x		   y		z	  colour
			{  from.x,  from.y,  from.z,   col},
			{  to.x,    to.y,    to.z,     col},
		};
		pDevice->SetTexture( 0, NULL);

		
#if(DIRECT3D_VERSION >= 0x0900)
		pDevice->SetFVF( TLVERTEX::FVF_VERTEX );
#else
		pDevice->SetVertexShader( FVF_VERTEX );
#endif // DIRECT3D_VERSION

		//static D3DXMATRIX matWorld = D3DXMATRIX(1,0,0,0,
		//										0,1,0,0,
		//										0,0,1,0,
		//										0,0,0,1);
		pDevice->SetTransform(D3DTS_WORLD, &mat);
		pDevice->DrawPrimitiveUP( D3DPT_LINELIST, 1, Vertex, sizeof( TLVERTEX ) );

}// End DrawLine(..)


/***************************************************************************/
/*                                                                         */
/*  Name: DrawCircle()                                                     */
/*                                                                         */
/*  Draws a 2D Circle in the 3D World around the X-Z Axis - you can apply  */
/*  a rotation so you can have it in any axis and any position in the      */
/*  world.                                                                 */
/*                                                                         */
/***************************************************************************/

bool DrawCircle(IDirect3DDevice9  * pDevice, 
				const D3DXVECTOR3& pos, 
				const D3DXMATRIX&  rot, 
				const D3DXVECTOR3& dim,
				unsigned int col0,  /*=0xffff0000*/
				unsigned int colhit /*=0xff222222*/,
				bool selectable, /*=true*/
				const D3DXVECTOR3& localrot /*=D3DXVECTOR3(0,0,0)*/)
{
	bool hit = false;

	if (selectable)
	{
		hit = 
		RayCircleHit(	pos, 
						rot, 
						dim,
						10.0f,
						localrot);
	}

	unsigned int col = col0;
	if (hit && selectable) 
	{
		col = colhit;
	}


  	struct TLVERTEX
	{
		float x,y,z;
		unsigned int colour;
		enum{ FVF_TLVERTEX = D3DFVF_XYZ | D3DFVF_DIFFUSE };
	};

	pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

	pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	pDevice->SetTextureStageState(0,D3DTSS_COLOROP,	D3DTOP_SELECTARG1);
	pDevice->SetTextureStageState(0,D3DTSS_COLORARG1,	D3DTA_DIFFUSE);


	const int SEGMENTS = 15;
	float step = (2.0f*D3DX_PI)/SEGMENTS;

	static TLVERTEX Vertex[40];

	D3DXMATRIX matLocRot;
	D3DXMatrixRotationYawPitchRoll(&matLocRot, localrot.y, localrot.x, localrot.z);

	D3DXMATRIX matMove, matScale, matRot, matWorld;
	D3DXMatrixTranslation(&matMove, pos.x, pos.y, pos.z);
	D3DXMatrixScaling(&matScale, dim.x, dim.y, dim.z);
	//D3DXMatrixRotationYawPitchRoll(&matRot, rot.y, rot.x, rot.z);
	matRot = rot;

	matWorld = matLocRot * matRot;/// * matScale * matMove;

	for (int i=0; i<SEGMENTS+1; i++)
	{
		float val = i*step;

		float x = cos(val)*3.0f;
		float z = sin(val)*3.0f;

		Vertex[i].x = x;
		Vertex[i].y = 0;
		Vertex[i].z = z;
		Vertex[i].colour = col;

		D3DXVec3TransformCoord((D3DXVECTOR3*)&Vertex[i], (D3DXVECTOR3*)&Vertex[i], &matWorld);
	}
	 

	pDevice->SetTexture( 0, NULL);

		
	#if(DIRECT3D_VERSION >= 0x0900)
		pDevice->SetFVF( TLVERTEX::FVF_TLVERTEX );
	#else
		pDevice->SetVertexShader( TLVERTEX::FVF_TLVERTEX );
	#endif // DIRECT3D_VERSION

		/*
	D3DXMATRIX matMove, matScale, matRot, matWorld;
	D3DXMatrixTranslation(&matMove, pos.x, pos.y, pos.z);
	D3DXMatrixScaling(&matScale, dim.x, dim.y, dim.z);
	D3DXMatrixRotationYawPitchRoll(&matRot, rot.y, rot.x, rot.z);

	matWorld = matRot * matScale * matMove;
	pDevice->SetTransform(D3DTS_WORLD, &matWorld);
	*/


	D3DXMATRIX matIdentity;
	D3DXMatrixIdentity(&matIdentity);
	pDevice->SetTransform(D3DTS_WORLD, &matIdentity);

	HRESULT hr = 
	pDevice->DrawPrimitiveUP( D3DPT_LINESTRIP, SEGMENTS, Vertex, sizeof( TLVERTEX ) );


	return hit;
}//End DrawCircle(..)


/***************************************************************************/
/*                                                                         */
/*  Name: RayCircleHit()                                                   */
/*                                                                         */
/*  Simple check function, so we can determine if a ray has hit our        */
/*  circles path, - Note, not a sphere check, a circles path, and we       */
/*  specify how close to the circles path.                                 */
/*                                                                         */
/***************************************************************************/

bool RayCircleHit(  const D3DXVECTOR3& pos, 
					const D3DXMATRIX&  rot, 
					const D3DXVECTOR3& dim,
					float checkBorder,
					const D3DXVECTOR3& localrot /*=D3DXVECTOR3(0,0,0)*/)
{
	const int SEGMENTS = 15;
	float step = (2.0f*D3DX_PI)/SEGMENTS;

	static D3DXVECTOR3 Vertex[40];

	D3DXMATRIX matLocRot;
	D3DXMatrixRotationYawPitchRoll(&matLocRot, localrot.y, localrot.x, localrot.z);

	D3DXMATRIX matMove, matScale, matRot, matWorld;
	D3DXMatrixTranslation(&matMove, pos.x, pos.y, pos.z);
	D3DXMatrixScaling(&matScale, dim.x, dim.y, dim.z);
	//D3DXMatrixRotationYawPitchRoll(&matRot, rot.y, rot.x, rot.z);
	matRot = rot;

	matWorld = matLocRot * matRot ;//* matScale * matMove;

	for (int i=0; i<SEGMENTS+1; i++)
	{
		float val = i*step;

		float x = cos(val)*3;
		float z = sin(val)*3;

		Vertex[i].x = x;
		Vertex[i].y = 0;
		Vertex[i].z = z;
		//Vertex[i].w = 1;

		//D3DXVec4Transform(&Vertex[i], &Vertex[i], &matWorld);
		D3DXVec3TransformCoord(&Vertex[i], &Vertex[i], &matWorld);
	}
	 

	extern IDirect3DDevice9* g_pD3DDevice;

	D3DXMATRIX matView, matProj, matProjT, matProjI, matViewI, matViewT;
	g_pD3DDevice->GetTransform( D3DTS_VIEW,			&matView);
	g_pD3DDevice->GetTransform( D3DTS_PROJECTION,	&matProj);
	D3DXMatrixTranspose(&matProjT, &matProj);
	D3DXMatrixTranspose(&matProjI, &matProj);
	D3DXMatrixInverse(&matViewI, NULL, &matView);
	D3DXMatrixTranspose(&matViewT, &matView);

	extern HWND	g_hwnd;
	RECT rect;
	GetClientRect(g_hwnd, &rect);

	float w = (float)rect.right;
	float h = (float)rect.bottom;

	D3DXMATRIX matViewProj = matView * matProj;

	D3DVIEWPORT9 viewData = { 0, 0, (DWORD)w, (DWORD)h, 0.0f, 1.0f };
	g_pD3DDevice->SetViewport(&viewData);



	POINT ptCursor;
	GetCursorPos(&ptCursor);
	extern HWND	g_hwnd;
	ScreenToClient(g_hwnd, &ptCursor);
	D3DXVECTOR2 vCursor((float)ptCursor.x, (float)ptCursor.y);

	bool allInside = false;

	for (int i=0; i<SEGMENTS; i++)
	{
		D3DXVECTOR4 p0 = Vertex[i];
		D3DXVECTOR4 p1 = Vertex[i+1];
		p0.w=p1.w=1.0f;

		D3DXVec4Transform(&p0, &p0, &matViewProj);
		D3DXVec4Transform(&p1, &p1, &matViewProj);
		p0 /= p0.w;
		p1 /= p1.w;


		p0.x = w*0.5f*(p0.x + 1.0f);
		p0.y = h*0.5f*(-p0.y + 1);
		p1.x = w*0.5f*(p1.x + 1);
		p1.y = h*0.5f*(-p1.y + 1);
		

		//D3DXMATRIX matIdentity;
		//D3DXMatrixIdentity(&matIdentity);
		//D3DXVECTOR3 a0, a1;
		//D3DXVec3Project(&a0, &Vertex[i],   &viewData, &matProj, &matView, &matIdentity);
		//D3DXVec3Project(&a1, &Vertex[i+1], &viewData, &matProj, &matView, &matIdentity);
		//D3DXVECTOR2 from = D3DXVECTOR2(a0.x, a0.y);
		//D3DXVECTOR2 to   = D3DXVECTOR2(a1.x, a1.y);

		D3DXVECTOR2 from = D3DXVECTOR2(p0.x, p0.y);
		D3DXVECTOR2 to   = D3DXVECTOR2(p1.x, p1.y);

		//DrawLine2D(g_pD3DDevice, from, to );

		D3DXVECTOR2 dir = to - from;
		D3DXVec2Normalize(&dir, &dir);
		D3DXVECTOR2 perpDir = D3DXVECTOR2(-dir.y, dir.x);


		DrawLine2D(g_pD3DDevice, from, from - checkBorder * perpDir );

		// 4 Corners
		D3DXVECTOR2 c[4];
		c[0] = (to+checkBorder*dir)   + checkBorder*perpDir;
		c[1] = (to+checkBorder*dir)   - checkBorder*perpDir;
		c[2] = (from-checkBorder*dir) - checkBorder*perpDir;
		c[3] = (from-checkBorder*dir) + checkBorder*perpDir;

		float t0 = (D3DXVec2CCW(&(c[3]-vCursor), &(c[0]-vCursor)));
		float t1 = (D3DXVec2CCW(&(c[0]-vCursor), &(c[1]-vCursor)));
		float t2 = (D3DXVec2CCW(&(c[1]-vCursor), &(c[2]-vCursor)));
		float t3 = (D3DXVec2CCW(&(c[2]-vCursor), &(c[3]-vCursor)));

		bool inside = true;
		inside &= ((t0) > 0) ? false : true;
		inside &= ((t1) > 0) ? false : true;
		inside &= ((t2) > 0) ? false : true;
		inside &= ((t3) > 0) ? false : true;

		
		allInside |= inside;
	}


	return allInside;

}//End DrawCircle(..)

/***************************************************************************/
/*                                                                         */
/*  Name: DrawText(...                                                     */
/*                                                                         */
/*  Draws a 2D Circle in the 3D World around the X-Z Axis - you can apply  */
/*  a rotation so you can have it in any axis and any position in the      */
/*  world.                                                                 */
/*                                                                         */
/***************************************************************************/

void DrawText(IDirect3DDevice9 * pDevice,
				char * str, 
				int xpos /*= 10*/, 
				int ypos /*= 10*/, 
				unsigned int rgb /*= 0xffffff*/,
				int size /*=15*/)
{
	class font_c
	{
		public:
		font_c(){pFont=NULL;}
		~font_c(){pFont->Release();}
		ID3DXFont* pFont;
	};
	static font_c font;

	ID3DXFont** pFont = &font.pFont;

#if(DIRECT3D_VERSION >= 0x0900)

	if (*pFont==NULL)
	{
	D3DXCreateFont(pDevice,						// LPDIRECT3DDEVICE9
					size,						// Width
					0,							// Height
					FW_NORMAL,					// Weight
					1,							// MipLevels
					false,						// Italic
					DEFAULT_CHARSET,			// CharSet
					OUT_DEFAULT_PRECIS,			// OutputPrecision
					ANTIALIASED_QUALITY,		// Quality
					FIXED_PITCH | FF_DONTCARE,	// PitchAndFamily
					"Arial",					// pFacename
					pFont);						// LPD3DXFONT
	}
#else
	static LOGFONT lfont={
			size, //height
			0,  //width; 
			0,  // lfEscapement; 
			0,  //lfOrientation; 
			FW_BOLD, // lfWeight; 
			FALSE, // lfItalic; 
			FALSE, // lfUnderline; 
			FALSE, // lfStrikeOut; 
			DEFAULT_CHARSET, // lfCharSet; 
			OUT_DEFAULT_PRECIS, //lfOutPrecision; 
			CLIP_DEFAULT_PRECIS, // lfClipPrecision; 
			ANTIALIASED_QUALITY,// lfQuality; 
			FIXED_PITCH,// lfPitchAndFamily; 
			"Arial"// lfFaceName[LF_FACESIZE]; 
			};

	lfont.lfHeight = size;

   //This is all there is to creating a D3DXFont.  Hmmmm....
   D3DXCreateFontIndirect( pDevice ,&lfont,&pFont );

#endif // DIRECT3D_VERSION



   static char szbuff[900];
   sprintf(szbuff, "%s", str );
   RECT rect = {xpos,ypos, ((int)xpos)+800,ypos+800};


#if(DIRECT3D_VERSION >= 0x0900)
   (*pFont)->DrawTextA(NULL,
						szbuff,
						-1,
						&rect,
						DT_LEFT,
						0xFFBBBBFF);
#else
   pFont->DrawTextA(szbuff,
						-1,
						&rect,
						DT_LEFT,
						0xFFBBBBFF);
#endif // DIRECT3D_VERSION


	//(*pFont)->Release();

}// End DrawText(..)




void DrawLine2D(  IDirect3DDevice9  * pDevice, 
				  D3DXVECTOR2 from,
				  D3DXVECTOR2 to,
				  unsigned int col /*=0xffffffff*/ )
{
		struct TLVERTEX
		{
			float x,y,z,rhw;
			unsigned int colour;
			enum { FVF_VERTEX = (D3DFVF_XYZRHW | D3DFVF_DIFFUSE) };
		};

		pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
		pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
		pDevice->SetTextureStageState(0,D3DTSS_COLOROP,	D3DTOP_SELECTARG1);
		pDevice->SetTextureStageState(0,D3DTSS_COLORARG1,	D3DTA_DIFFUSE);
		
		TLVERTEX Vertex[2] = 
		{
			//   x			  y		 z   rhw    colour
			{    from.x,   from.y,   0,  1.0f,   col},
			{    to.x,     to.y,     0,  1.0f,   col},
		};
		pDevice->SetTexture( 0, NULL);

		
#if(DIRECT3D_VERSION >= 0x0900)
		pDevice->SetFVF( TLVERTEX::FVF_VERTEX );
#else
		pDevice->SetVertexShader( FVF_VERTEX );
#endif // DIRECT3D_VERSION

		D3DXMATRIX matWorld = D3DXMATRIX(1,0,0,0,
					  					 0,1,0,0,
										 0,0,1,0,
										 0,0,0,1);
		pDevice->SetTransform(D3DTS_WORLD, &matWorld);

		pDevice->DrawPrimitiveUP( D3DPT_LINELIST, 1, Vertex, sizeof( TLVERTEX ) );
}// End DrawLine2D(..)

















